using System.Text.Json;
using System.Text.Json.Serialization;
using System.Text.Json.Serialization.Metadata;

using Blazored.LocalStorage;

using Microsoft.Extensions.DependencyInjection;

using NSeguin.Dev.Web.Services;

namespace NSeguin.Dev.Web.Extensions;

public static class ServiceCollectionExtensions
{
    public static IServiceCollection AddNSeguinDevWebServer(this IServiceCollection services)
    {
        return AddNSeguinDevWebCommon(services, false);
    }

    public static IServiceCollection AddNSeguinDevWebClient(this IServiceCollection services)
    {
        return AddNSeguinDevWebCommon(services, true);
    }

    private static IServiceCollection AddNSeguinDevWebCommon(
        this IServiceCollection services,
        bool isWebAssembly)
    {
        AddAppDataService(services, isWebAssembly);
        return services;
    }

    private static void AddAppDataService(IServiceCollection services, bool isWebAssembly)
    {
        if (isWebAssembly)
        {
            services.AddBlazoredLocalStorageAsSingleton(
                config =>
                {
                    ConfigureJsonSerializerOptionsForBlazoredLocalStorage(
                        config.JsonSerializerOptions);
                });

            services.AddSingleton<IAppDataService, AppDataService>();
        }
        else
        {
            services.AddBlazoredLocalStorage(
                config =>
                {
                    ConfigureJsonSerializerOptionsForBlazoredLocalStorage(
                        config.JsonSerializerOptions);
                });

            services.AddScoped<IAppDataService, AppDataService>();
        }
    }

    private static void ConfigureJsonSerializerOptionsForBlazoredLocalStorage(
        this JsonSerializerOptions options)
    {
        options.WriteIndented = true;
        options.PropertyNamingPolicy = JsonNamingPolicy.CamelCase;
        options.DictionaryKeyPolicy = JsonNamingPolicy.CamelCase;
        options.DefaultIgnoreCondition = JsonIgnoreCondition.WhenWritingNull;
        options.Converters.Add(new JsonStringEnumConverter(JsonNamingPolicy.CamelCase));
        options.TypeInfoResolverChain.Insert(0, AppDataSerializerContext.Default);
        options.TypeInfoResolverChain.Add(new DefaultJsonTypeInfoResolver());
    }
}
