using System.Diagnostics.CodeAnalysis;

using UserConfiguration = NSeguin.Dev.Web.UserConfiguration;

namespace NSeguin.Dev.Web.Services;

public interface IAppDataService
{
    event AsyncEventHandler? Initialized;

    [MemberNotNullWhen(true, nameof(UserConfiguration))]
    bool IsInitialized { get; }

    UserConfiguration? UserConfiguration { get; }

    /// <inheritdoc cref="DoOnceInitializedAsync(Func{ValueTask})" />
    ValueTask DoOnceInitializedAsync(AsyncEventHandler action);

    /// <summary>
    ///     Runs the action when the service is initialized. If the service is already initialized, the action is run
    ///     immediately.
    /// </summary>
    /// <param name="action">The action to run.</param>
    /// <returns>A <see cref="ValueTask" /> representing the asynchronous operation.</returns>
    ValueTask DoOnceInitializedAsync(Func<ValueTask> action);

    ValueTask InitializeAsync();

    ValueTask SaveAsync();
}
