using System.Collections.Concurrent;
using System.Text.RegularExpressions;

using Microsoft.AspNetCore.Components;
using Microsoft.AspNetCore.Components.Rendering;

namespace NSeguin.Dev.Web.Components.Shared;

public partial class HeroIcon : ComponentBase
{
    private const string PathBase = "wwwroot/lib/heroicons";
    private static readonly ConcurrentDictionary<string, string> Cache = new();

    private Task<string>? _svgTask;

    [Parameter]
    public HeroIconKind? Kind { get; set; }

    public bool Outline { get; set; }

    public bool Small { get; set; }

    private string? RawSvg { get; set; }

    protected override void BuildRenderTree(RenderTreeBuilder builder)
    {
        builder.AddMarkupContent(0, RawSvg);
    }

    protected override void OnParametersSet()
    {
        if (Small && Outline)
        {
            throw new InvalidOperationException(
                "Small and Outline cannot be true at the same time.");
        }

        if (Kind is null)
        {
            throw new InvalidOperationException("Kind cannot be null.");
        }

        _svgTask = (Outline, Small) switch
        {
            (true, false) => Kind.Outline,
            (false, true) => Kind.Small,
            (false, false) => Kind.Solid,
            _ => throw new InvalidOperationException("Invalid combination of parameters.")
        };

        if (_svgTask.IsCompletedSuccessfully)
        {
            RawSvg = _svgTask.Result;
        }
    }

    protected override async Task OnInitializedAsync()
    {
        if (Kind is null)
        {
            throw new InvalidOperationException("Kind cannot be null.");
        }

        if (RawSvg is null && _svgTask is not null)
        {
            RawSvg = await _svgTask;
            StateHasChanged();
        }
    }
}
